@@ -141,18 +141,13 @@ class AsContainer:
         """
         raise AssertionError
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         """
         Args:
             file (:doc:`uproot.reading.CommonFileMethods`): The file associated
                 with this interpretation's ``TBranch``.
+            context (dict): Context for the Form-generation; defaults are
+                the remaining arguments below.
             index_format (str): Format to use for indexes of the
                 ``awkward.forms.Form``; may be ``"i32"``, ``"u32"``, or
                 ``"i64"``.
@@ -284,23 +279,14 @@ class AsDynamic(AsContainer):
         else:
             return uproot.model.classname_decode(self._model.__name__)[0]
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         awkward = uproot.extras.awkward()
         if self._model is None:
             raise uproot.interpretation.objects.CannotBeAwkward("dynamic type")
         else:
             return awkward.forms.ListOffsetForm(
-                index_format,
-                uproot._util.awkward_form(
-                    self._model, file, index_format, header, tobject_header, breadcrumbs
-                ),
+                context["index_format"],
+                uproot._util.awkward_form(self._model, file, context),
                 parameters={"uproot": {"as": "array", "header": self._header}},
             )
 
@@ -341,14 +327,7 @@ class AsFIXME(AsContainer):
     def typename(self):
         return "unknown"
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         raise uproot.interpretation.objects.CannotBeAwkward(self.message)
 
     def read(self, chunk, cursor, context, file, selffile, parent, header=True):
@@ -426,17 +405,10 @@ class AsString(AsContainer):
         else:
             return self._typename
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         awkward = uproot.extras.awkward()
         return awkward.forms.ListOffsetForm(
-            index_format,
+            context["index_format"],
             awkward.forms.NumpyForm((), 1, "B", parameters={"__array__": "char"}),
             parameters={
                 "__array__": "string",
@@ -540,14 +512,7 @@ class AsPointer(AsContainer):
         else:
             return _content_typename(self._pointee) + "*"
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         raise uproot.interpretation.objects.CannotBeAwkward("arbitrary pointer")
 
     def read(self, chunk, cursor, context, file, selffile, parent, header=True):
@@ -630,22 +595,13 @@ class AsArray(AsContainer):
         shape = "".join(f"[{d}]" for d in self.inner_shape)
         return _content_typename(self._values) + "[]" + shape
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         awkward = uproot.extras.awkward()
-        values_form = uproot._util.awkward_form(
-            self._values, file, index_format, header, tobject_header, breadcrumbs
-        )
+        values_form = uproot._util.awkward_form(self._values, file, context)
         for dim in reversed(self.inner_shape):
             values_form = awkward.forms.RegularForm(values_form, dim)
         return awkward.forms.ListOffsetForm(
-            index_format,
+            context["index_format"],
             values_form,
             parameters={
                 "uproot": {
@@ -719,6 +675,152 @@ in file {}""".format(
                 return uproot._util.objectarray1d(out).reshape(-1, *self.inner_shape)
 
 
+class AsRVec(AsContainer):
+    """
+    Args:
+        header (bool): Sets the :ref:`uproot.containers.AsContainer.header`.
+        values (:doc:`uproot.model.Model` or :doc:`uproot.containers.Container`): Data
+            type for data nested in the container.
+
+    A :doc:`uproot.containers.AsContainer` for ``ROOT::VecOps::RVec``.
+    """
+
+    def __init__(self, header, values):
+        self.header = header
+        if isinstance(values, AsContainer):
+            self._values = values
+        elif isinstance(values, type) and issubclass(
+            values, (uproot.model.Model, uproot.model.DispatchByVersion)
+        ):
+            self._values = values
+        else:
+            self._values = numpy.dtype(values)
+
+    def __hash__(self):
+        return hash((AsRVec, self._header, self._values))
+
+    @property
+    def values(self):
+        """
+        Data type for data nested in the container.
+        """
+        return self._values
+
+    def __repr__(self):
+        if isinstance(self._values, type):
+            values = self._values.__name__
+        else:
+            values = repr(self._values)
+        return f"AsRVec({self._header}, {values})"
+
+    @property
+    def cache_key(self):
+        return f"AsRVec({self._header},{_content_cache_key(self._values)})"
+
+    @property
+    def typename(self):
+        return f"RVec<{_content_typename(self._values)}>"
+
+    def awkward_form(self, file, context):
+        awkward = uproot.extras.awkward()
+        return awkward.forms.ListOffsetForm(
+            context["index_format"],
+            uproot._util.awkward_form(self._values, file, context),
+            parameters={"uproot": {"as": "RVec", "header": self._header}},
+        )
+
+    def read(self, chunk, cursor, context, file, selffile, parent, header=True):
+        if self._header and header:
+            start_cursor = cursor.copy()
+            (
+                num_bytes,
+                instance_version,
+                is_memberwise,
+            ) = uproot.deserialization.numbytes_version(chunk, cursor, context)
+        else:
+            is_memberwise = False
+
+        # Note: self._values can also be a NumPy dtype, and not necessarily a class
+        # (e.g. type(self._values) == type)
+        _value_typename = _content_typename(self._values)
+        if is_memberwise:
+            if not issubclass(self._values, uproot.model.DispatchByVersion):
+                raise NotImplementedError(
+                    """streamerless memberwise serialization of class {}({})
+    in file {}""".format(
+                        type(self).__name__, _value_typename, selffile.file_path
+                    )
+                )
+
+            # uninterpreted header
+            cursor.skip(6)
+
+            length = cursor.field(chunk, _stl_container_size, context)
+
+            # no known class version number (maybe in that header? unclear...)
+            model = self._values.new_class(file, "max")
+
+            values = numpy.empty(length, dtype=_stl_object_type)
+
+            # only do anything if we have anything to read...
+            if length > 0:
+                for i in range(length):
+                    values[i] = model.read(
+                        chunk,
+                        cursor,
+                        dict(context, reading=False),
+                        file,
+                        selffile,
+                        parent,
+                    )
+
+                # memberwise reading!
+                for member_index in range(len(values[0].member_names)):
+                    for i in range(length):
+                        values[i].read_member_n(
+                            chunk, cursor, context, file, member_index
+                        )
+        else:
+            length = cursor.field(chunk, _stl_container_size, context)
+
+            values = _read_nested(
+                self._values, length, chunk, cursor, context, file, selffile, parent
+            )
+
+        out = ROOTRVec(values)
+
+        if self._header and header:
+            uproot.deserialization.numbytes_check(
+                chunk,
+                start_cursor,
+                cursor,
+                num_bytes,
+                self.typename,
+                context,
+                file.file_path,
+            )
+
+        return out
+
+    def __eq__(self, other):
+        if not isinstance(other, AsRVec):
+            return False
+
+        if self.header != other.header:
+            return False
+
+        if isinstance(self.values, numpy.dtype) and isinstance(
+            other.values, numpy.dtype
+        ):
+            return self.values == other.values
+        elif not isinstance(self.values, numpy.dtype) and not isinstance(
+            other.values, numpy.dtype
+        ):
+            return self.values == other.values
+        else:
+            return False
+
+
 class AsVector(AsContainer):
     """
     Args:
@@ -765,20 +867,11 @@ class AsVector(AsContainer):
     def typename(self):
         return f"std::vector<{_content_typename(self._values)}>"
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         awkward = uproot.extras.awkward()
         return awkward.forms.ListOffsetForm(
-            index_format,
-            uproot._util.awkward_form(
-                self._values, file, index_format, header, tobject_header, breadcrumbs
-            ),
+            context["index_format"],
+            uproot._util.awkward_form(self._values, file, context),
             parameters={"uproot": {"as": "vector", "header": self._header}},
         )
 
@@ -929,20 +1022,11 @@ class AsSet(AsContainer):
     def typename(self):
         return f"std::set<{_content_typename(self._keys)}>"
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         awkward = uproot.extras.awkward()
         return awkward.forms.ListOffsetForm(
-            index_format,
-            uproot._util.awkward_form(
-                self._keys, file, index_format, header, tobject_header, breadcrumbs
-            ),
+            context["index_format"],
+            uproot._util.awkward_form(self._keys, file, context),
             parameters={
                 "__array__": "set",
                 "uproot": {"as": "set", "header": self._header},
@@ -1087,35 +1171,14 @@ class AsMap(AsContainer):
             _content_typename(self._keys), _content_typename(self._values)
         )
 
-    def awkward_form(
-        self,
-        file,
-        index_format="i64",
-        header=False,
-        tobject_header=True,
-        breadcrumbs=(),
-    ):
+    def awkward_form(self, file, context):
         awkward = uproot.extras.awkward()
         return awkward.forms.ListOffsetForm(
-            index_format,
+            context["index_format"],
             awkward.forms.RecordForm(
                 (
-                    uproot._util.awkward_form(
-                        self._keys,
-                        file,
-                        index_format,
-                        header,
-                        tobject_header,
-                        breadcrumbs,
-                    ),
-                    uproot._util.awkward_form(
-                        self._values,
-                        file,
-                        index_format,
-                        header,
-                        tobject_header,
-                        breadcrumbs,
-                    ),
+                    uproot._util.awkward_form(self._keys, file, context),
+                    uproot._util.awkward_form(self._values, file, context),
                 )
             ),
             parameters={
@@ -1236,6 +1299,64 @@ class Container:
         raise AssertionError
 
 
+class ROOTRVec(Container, Sequence):
+    """
+    Args:
+        values (``numpy.ndarray`` or iterable): Contents of the ``ROOT::VecOps::RVec``.
+
+    Representation of a C++ ``ROOT::VecOps::RVec`` as a Python ``Sequence``.
+    """
+
+    def __init__(self, values):
+        if isinstance(values, types.GeneratorType):
+            values = numpy.asarray(list(values))
+        elif isinstance(values, Set):
+            values = numpy.asarray(list(values))
+        elif isinstance(values, (list, tuple)):
+            values = numpy.asarray(values)
+
+        self._values = values
+
+    def __str__(self, limit=85):
+        def tostring(i):
+            return _tostring(self._values[i])
+
+        return _str_with_ellipsis(tostring, len(self), "[", "]", limit)
+
+    def __repr__(self, limit=85):
+        return "<ROOTRVec {} at 0x{:012x}>".format(
+            self.__str__(limit=limit - 30), id(self)
+        )
+
+    def __getitem__(self, where):
+        return self._values[where]
+
+    def __len__(self):
+        return len(self._values)
+
+    def __contains__(self, what):
+        return what in self._values
+
+    def __iter__(self):
+        return iter(self._values)
+
+    def __reversed__(self):
+        return ROOTRVec(self._values[::-1])
+
+    def __eq__(self, other):
+        if isinstance(other, ROOTRVec):
+            return self._values == other._values
+        elif isinstance(other, Sequence):
+            return self._values == other
+        else:
+            return False
+
+    def tolist(self):
+        return [
+            x.tolist() if isinstance(x, (Container, numpy.ndarray)) else x for x in self
+        ]
+
+
 class STLVector(Container, Sequence):
     """
     Args:
