@@ -122,14 +122,18 @@ class AsStrings(uproot.interpretation.Interpretation):
     def awkward_form(
         self,
         file,
+        context=None,
         index_format="i64",
         header=False,
         tobject_header=True,
         breadcrumbs=(),
     ):
+        context = self._make_context(
+            context, index_format, header, tobject_header, breadcrumbs
+        )
         awkward = uproot.extras.awkward()
         return awkward.forms.ListOffsetForm(
-            index_format,
+            context["index_format"],
             awkward.forms.NumpyForm((), 1, "B", parameters={"__array__": "char"}),
             parameters={
                 "__array__": "string",
