@@ -4,7 +4,6 @@
 This module defines a versionless model for ``THashList``.
 """
 
-from __future__ import absolute_import
 
 import uproot
 
@@ -20,8 +19,8 @@ class Model_THashList(uproot.model.Model):
     def read_members(self, chunk, cursor, context, file):
         if self.is_memberwise:
             raise NotImplementedError(
-                """memberwise serialization of {0}
-in file {1}""".format(
+                """memberwise serialization of {}
+in file {}""".format(
                     type(self).__name__, self.file.file_path
                 )
             )
@@ -41,8 +40,8 @@ in file {1}""".format(
         if self.class_version is None:
             version = ""
         else:
-            version = " (version {0})".format(self.class_version)
-        return "<{0}{1} of {2} items at 0x{3:012x}>".format(
+            version = f" (version {self.class_version})"
+        return "<{}{} of {} items at 0x{:012x}>".format(
             self.classname,
             version,
             len(self),
